% Simple Convert from Neurolucida ASCII files to NEURON.hoc
% 
% usage: asctohoc(filename,showplot)  (ex. test.asc)
%
% ex. asctohoc(test.hoc,1)  will also plot the cell to check for errors
%
% output: filename.hoc (ex. test.hoc)
% contains code from convert.m by Steve Cox and Jay Raol

function asctohoc(filename,showplot)

OPENF = fopen(filename,'rt');

if OPENF == -1 
    disp(['failed to open file ' filename]);
    return
end

disp('storing coordinates...');

% Loops through the lines and Find Soma Data
word = ''; 		            % word on a line

while ~strcmp(word,'CellBody')  
    line = fgetl(OPENF);
    word = getword(line);
end

line = fgetl(OPENF);
word = getword(line);

isNumber = 0;
soma.x = [];
soma.y = [];
soma.z = [];

while ~strcmp(word,'Endofcontour'),     % loops through cellbody data
   
   num = 0;
   number = '';
   
   for j=1:length(line),	            % goes through each character in a line
      
      if line(j) ~= ' ' & line(j) ~= '(' & num <= 3, 
                                        % the character is not a space or '(' then it is a number 
                                        % and stored in number
     	 number = [number line(j)];
         isNumber = 1; 
      elseif isNumber,				
                                        % if a number has just been read in and the character 
                                        % is not a number, then reset
         num = num + 1;
         isNumber = 0;
         storeword = 1;
      end
         
      if num == 1 & storeword,          %store x,y,z information
         soma.x = [soma.x str2num(number)];
         number = '';
         storeword = 0;
      elseif num == 2 & storeword,
         soma.y = [soma.y str2num(number)];
         number = '';
         storeword = 0;
      elseif num == 3 & storeword,
         soma.z = [soma.z str2num(number)];
         number = '';
         storeword = 0;
      end      

   end	% end for j
   
   line = fgetl(OPENF);
   word = getword(line);

end  % end while

% Calculate Soma Properties (maximum distance between 2 traced points)

pcount = 1;

for j = 1:length(soma.x),
    for i = 1:length(soma.y),
        diams(pcount) = sqrt((soma.y(j)-soma.y(i))^2+(soma.x(j)-soma.y(i))^2);   
        pcount = pcount + 1;    
    end
end

soma.d = max(diams);
soma.l = soma.d;

% begin to read in dendrite information

line = fgetl(OPENF);           
word = getword(line);
while ~strcmp(word,'Dendrite')
   line = fgetl(OPENF);
   word = getword(line);
end

segcount = 0;                   

while ~feof(OPENF),            % loop until the end of file
    line = fgetl(OPENF);
    isNumber = 0;
    goloop = 1;
    
                                % check segment data
    if findstr(line,'; R') > 0, % if the line of data has 'R', as in the R-1-2, etc...
        goloop = 1;             % then we have found a segment chunk
        segcount = segcount + 1;
       
        segx = [];
        segy = [];
        segz = [];
        segd = [];
        connstr = [];
    else
        goloop = 0;
    end
    
    coordcount = 1;

    while goloop == 1,          % loops through each segment
   
        num = 0;
        number = '';
        
        
        for j=1:length(line),	% goes through each character in a line
      
            if line(j) ~= ' ' & line(j) ~= '(' & line(j) ~= ')' & num <= 4, 
                                % the character is not a space or '(' then it is a number 
                                % and stored in number
     	        number = [number line(j)];
                isNumber = 1; 
            elseif isNumber			
                                % if a number has just been read in and the character 
                                % is not a number, then reset
                num = num + 1;
                isNumber = 0;
                storeword = 1;
            end
         
            if num == 1 & storeword,  %store x coordinates in x vector, y in y vector
                segx = [segx str2num(number)];
                number = '';
                storeword = 0;
            elseif num == 2 & storeword,
                segy = [segy str2num(number)];
                number = '';
                storeword = 0;
            elseif num == 3 & storeword,
                segz = [segz str2num(number)];
                number = '';
                storeword = 0;
            elseif num == 4 & storeword,
                segd = [segd str2num(number)];
                number = '';
                storeword = 0;
            end

        end	
        
        if coordcount == 1,          % get the connectivity pattern
            j = 3;
            tempstr = line(length(line)-j);
            while tempstr ~= 'R',
                tempstr = line(length(line)-j);
                j = j+1;
            end    
            connstr = line(length(line)-j:length(line)-3);
        end
   
        line = fgetl(OPENF);
       
        coordcount = coordcount + 1;
        
        % check if we are still inside a segment chunk
        if findstr(line,'R') > 0
            goloop = 1;
        else
            goloop = 0;
        end
        
    end  % end while
    
    segdata = [segx; segy; segz; segd]';
    celldata{segcount} = segdata;
    conndata{segcount} = connstr;
%     conndata = connstr;
%     disp(['segment: ' num2str(segcount)]);
    
end

% close file
fclose(OPENF);

for j =1:length(conndata)
    if conndata{j}(1) == ' ',
        conndata{j}(1) = [];
    end
end

% Main HOC %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

filename(length(filename)-3:length(filename)) = [];
% WRITEHOC = fopen([filename '.hoc'],'w');
% 
% fprintf(WRITEHOC,'load_file("nrngui.hoc")');
% fprintf(WRITEHOC,'\n');
% fprintf(WRITEHOC,['xopen("' filename  '-morph.hoc")']);
% fprintf(WRITEHOC,'\n');
% fprintf(WRITEHOC,['xopen("' filename  '-mech.hoc")']);
% fprintf(WRITEHOC,'\n');
% % fprintf(WRITEHOC,['xopen("' filename  '-elect.hoc")']);
% % fprintf(WRITEHOC,'\n');
% 
% fclose(WRITEHOC);
% disp([filename '.hoc written']);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Mophology %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
WRITEMORPH = fopen([filename '-morph.hoc'],'w');

% write the definitions

fprintf(WRITEMORPH,['ndend = ' num2str(segcount)]);
fprintf(WRITEMORPH,'\n');
fprintf(WRITEMORPH,'create soma, dend[ndend]');
fprintf(WRITEMORPH,'\n');
fprintf(WRITEMORPH,'access soma');
fprintf(WRITEMORPH,'\n');
fprintf(WRITEMORPH,'\n');

% write the connections: roots to soma NEEDS TO BE CLARIFIED

treecount = 1;
for j = 1:segcount,
    if conndata{j} == 'R',
        fprintf(WRITEMORPH,['connect dend[' num2str(j-1) '](0), soma(0.5)']);   
        fprintf(WRITEMORPH,'\n');
        roots(treecount) = j;
        treecount = treecount + 1;
    end
end

fprintf(WRITEMORPH,'\n');

% write the branch connections (using string search, not base 3 conversion)

roots(length(roots)+1) = segcount;
a = 1;
h = waitbar(0,'Writing connections...');
while a < length(roots),
    
    j = roots(a)+1;
    
    if a == length(roots)-1,
        cond = segcount+1;
    else    
        cond = roots(a+1);
    end
    
    while j < cond,
    
        b = 1;
        lastdash = 0;
        
        while ~lastdash,
            if conndata{j}(length(conndata{j})-b) == '-',
                lastdash = length(conndata{j})-b;
            else
                b = b+1;
            end
        end
       
        k = roots(a);
        while k < roots(a+1),
            if length(conndata{k}) == length(conndata{j}(1:lastdash-1)),
                if conndata{k} == conndata{j}(1:lastdash-1),
                    fprintf(WRITEMORPH,['connect dend[' num2str(j-1) '](0), dend[' num2str(k-1) '](1)']);      
                    fprintf(WRITEMORPH,'\n');
                    
                    % simulates "multiple clicking on branch points"
                    celldata{j}(2:size(celldata{j},1)+1,:) = celldata{j}(1:size(celldata{j},1),:);
                    celldata{j}(1,:) = celldata{k}(size(celldata{k},1),:);
                end
            end
            k = k + 1;
        end
        j = j + 1;
    end
    a = a + 1;
    fprintf(WRITEMORPH,'\n');
    waitbar(a/length(roots),h)
end    
close(h)

% to specify how many compartments are in each big segment

% for j = 1:segcount
%     nsegdata(j) = size(celldata{j},1);
% end

% write the soma coordinates

fprintf(WRITEMORPH,'\n');
fprintf(WRITEMORPH,'soma {');

fprintf(WRITEMORPH,'\n');
fprintf(WRITEMORPH,'\t');
fprintf(WRITEMORPH,['pt3dadd(' num2str(celldata{1}(1,1)) ',' num2str(celldata{1}(1,2)) ',' ...
                           num2str(celldata{1}(1,3)-0.01) ',' num2str(celldata{1}(1,4)) ')']);
fprintf(WRITEMORPH,'\n');
fprintf(WRITEMORPH,'\t');
fprintf(WRITEMORPH,['pt3dadd(' num2str(celldata{1}(1,1)) ',' num2str(celldata{1}(1,2)) ',' ...
                           num2str(celldata{1}(1,3)+0.01) ',' num2str(celldata{1}(1,4)) ')']);
fprintf(WRITEMORPH,'\n');

% fprintf(WRITEMORPH,'\n');
% fprintf(WRITEMORPH,'\t');
% fprintf(WRITEMORPH,'Ra = 123');
fprintf(WRITEMORPH,'\n');
fprintf(WRITEMORPH,'\t');
fprintf(WRITEMORPH,['diam = ' num2str(soma.d)]);
fprintf(WRITEMORPH,'\n');
fprintf(WRITEMORPH,'\t');
fprintf(WRITEMORPH,['L = ' num2str(soma.l)]);
fprintf(WRITEMORPH,'\n');

fprintf(WRITEMORPH,'}');
fprintf(WRITEMORPH,'\n');

h = waitbar(0,'Writing coordinates...');
for j = 1:segcount
    treesegcount(j) = size(celldata{j},1);
    
    fprintf(WRITEMORPH,'\n');
    fprintf(WRITEMORPH,['dend[' num2str(j-1) '] {']);
    
    for k = 1:treesegcount(j)
        fprintf(WRITEMORPH,'\n');
        fprintf(WRITEMORPH,'\t');
        fprintf(WRITEMORPH,['pt3dadd(' num2str(celldata{j}(k,1)) ',' num2str(celldata{j}(k,2)) ',' ...
                                   num2str(celldata{j}(k,3)) ',' num2str(celldata{j}(k,4)) ')']);
    end
    
%     fprintf(WRITEMORPH,'\n');
%     fprintf(WRITEMORPH,'\t');
%     fprintf(WRITEMORPH,['nseg = ' num2str(nsegdata(j))]);
    fprintf(WRITEMORPH,'\n');
    fprintf(WRITEMORPH,'}');
    waitbar(j/segcount,h);
end
close(h);

fclose(WRITEMORPH);

treecount = treecount - 1;

disp(['Trees: ' num2str(treecount)]);

disp([filename '-morph.hoc written']);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Mechanisms %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% WRITEMECH = fopen([filename '-mech.hoc'],'w');
% 
% fprintf(WRITEMECH,'soma {');
% fprintf(WRITEMECH,'\n');
% fprintf(WRITEMECH,'\t');
% fprintf(WRITEMECH,'insert hh');
% fprintf(WRITEMECH,'\n');
% fprintf(WRITEMECH,'}');
% fprintf(WRITEMECH,'\n');
% 
% for j = 1:segcount
%     fprintf(WRITEMECH,['dend[' num2str(j-1) '] {']);
%     fprintf(WRITEMECH,'\n');
%     fprintf(WRITEMECH,'\t');
%     fprintf(WRITEMECH,'insert pas');   % active dendrites
% %     fprintf(WRITEMECH,'\n');
% %     fprintf(WRITEMECH,'\t');
% %     fprintf(WRITEMECH,'Ra = 123');
%     fprintf(WRITEMECH,'\n');
%     fprintf(WRITEMECH,'}');
%     fprintf(WRITEMECH,'\n');
% end
% 
% fclose(WRITEMECH);
% disp([filename '-mech.hoc written']);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Electrodes %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% WRITEELECT = fopen([filename '-elect.hoc'],'w');
% 
% fprintf(WRITEELECT,'objectvar stim');
% fprintf(WRITEELECT,'\n');
% fprintf(WRITEELECT,'soma stim = new IClamp(0.5)');  % center of soma
% fprintf(WRITEELECT,'\n');
% fprintf(WRITEELECT,'stim.del = 50');
% fprintf(WRITEELECT,'\n');
% fprintf(WRITEELECT,'stim.dur = 200');
% fprintf(WRITEELECT,'\n');
% fprintf(WRITEELECT,'stim.amp = 1.5');        % nA in NEURON
% fprintf(WRITEELECT,'\n');
% fprintf(WRITEELECT,'tstop = 300');
% 
% fclose(WRITEELECT);
% disp([filename '-elect.hoc written']);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% if showplot == 1
%     plotcell(segcount,celldata,soma);
% end
    
function  a = area(x,y)

% AREA(X,Y)  Calculates the area of the 2-dimensional polygon
%      formed by vertices with coordinates vectors  X and Y.
%   Kirill K. Pankratov, kirill@plume.mit.edu.
%   April 20, 1994

x = [x(:); x(1)]; y = [y(:); y(1)];
plot(x,y,'k')
hold on
lx = length(x);
a = abs( (x(2:lx)-x(1:lx-1))' * (y(1:lx-1)+y(2:lx)) )/2;

return

function word = getword(line)
word = '';
for j=1:length(line),
   if line(j) == '"',
      word = '';
      return;
   end
   if isletter(line(j)),
      word = [word line(j)];
   end
end
return

function plotcell(segcount,celldata,soma)

%Plottage
 
close all;
hold on;
axis off;
axis equal;

for i = 1:segcount
    for j = 1:size(celldata{i},1)-1
        plot3([celldata{i}(j,1) celldata{i}(j+1,1)],...
              [celldata{i}(j,2) celldata{i}(j+1,2)],...
              [celldata{i}(j,3) celldata{i}(j+1,3)],'k-');
    end
end
for i = 1:length(soma.x)-1
    plot3([soma.x(i) soma.x(i+1)],[soma.y(i) soma.y(i+1)],[soma.z(i) soma.z(i+1)],'r-')
end