clear
close all
clc

% read in data

filename = 'AR-1-20-04-B.asc'

%filename = 'test.asc'

nc = neur_converterh(filename);

% simulation time

simd.dt = 0.01;   % ms
simd.T = 10;    % ms

simd.Is.start = 15;  % ms
simd.Is.end = 40;   % ms

% current pulse

simd.Is.mag = 1e-5;  % uA
%simd.Is.mag = 0;  % nA

simd.t = 0:simd.dt:simd.T;

Nt = simd.T/simd.dt+1;

% construct morphology

md.grid.rat = 3;
md = neur_loadmorph2(nc,50,md.grid.rat)

% parameters

p_const = neur_param_const(nc);
p_conds = neur_param_conds(nc,md);

plot(p_conds.pas.Gm{1})
title('actual conductance')

% M K L

[M, K, L, b, md] = neur_assembleMKh(nc,md,p_const,p_conds);

save ncmd nc md

% Hines solver

disp('M,K,L ready')
disp('Solving system...')

% solve for v

tic
[v,t] = neur_solverh(M,K,L,b,md,p_const,p_conds,simd,filename);
toc

VMEAS = neur_get_recdata(md,filename);

%pm.pas.Gm{1} = 0.15*ones(1,md.interp.gsize(1));


    pm.pas.Gm = mean(p_conds.pas.CGm{1})*ones(length(md.grid.Cxmid{1}),1);


% `unravel' the conductance cell array for fminunc 
% 
% tic
% [obj,Gval] = neur_femfun(pm.pas.Gm,nc,md,p_const,simd,VMEAS,filename);
% toc

options = optimset('display','iter',...
                   'maxiter',500,...
                   'tolfun',1e-3,...
                   'derivativecheck','on',...
                   'gradobj','on',...
                   'largescale','off');

rG = fminunc('neur_femfun',pm.pas.Gm,options,nc,md,p_const,...
                                  simd,VMEAS,filename);

hold                              
temp = neur_ctof(neur_cellularize(rG,md),md);
plot(temp{1})
