% read in data

clear 
close all

filename = 'hinesfork.asc'

nc = neur_converterh(filename);

% simulation time

simd.dt = 0.05;   % ms
simd.T = 60;    % ms

simd.Is.start = 15;  % ms
simd.Is.end = 40;   % ms

% current pulse

simd.Is.mag = 0.05e-3;  % nA

simd.t = 0:simd.dt:simd.T;

k = 1;

for ii = 1:1

    % construct morphology

    md = neur_loadmorph(nc,ii);

    % parameters

    pm = neur_get_params(nc,md);

    % M K L

    [M, K, L, md] = neur_assembleMKh(nc,md,pm);
    
    md.count.L
    
    % save ncmd nc md;

    % construct LHS
    
    LHS = M+simd.dt*(L+K);
    RHS = zeros(md.count.L,1);
    
    % construct RHS
    
    RHS(end) = 0.00003;

    % Hines solver
    
    tic
    [A,b,val] = hines(LHS,RHS,nc,md);
    
    % time it took
    ts(k) = toc;
    % matrix size
    ms(k) = md.count.L;
   
    % Matlab backslash
    
    val2 = LHS\RHS;
   
    
    
    % max error
    maxerr(k) = max(val'-val2);
    k = k + 1;
    
    
end

subplot(2,1,1)
plot(ms,ts)
xlabel('matrix size NxN')
ylabel('time (s)')
subplot(2,1,2)
plot(ms,maxerr)
xlabel('matrix size NxN')
ylabel('max error')

% disp('M,K,L ready')
% 
% disp('Solving system...')
% 
% % solve for v
% 
% [v,t] = neur_solverh(M,K,L,md,pm,simd,filename);
% 
% fprintf('\n')