function neur_cellguih(action)

old_format=get(0,'Format');
if nargin<1,
   action='initialize';
end;

if strcmp(action,'initialize'),
   oldFigNumber=watchon;
   
   figNumber=figure( ...
      'Name','Cell Viewer', ...
      'NumberTitle','off', ...
      'Visible','off', ...
      'Color','black',...
      'BackingStore','off');
   axes( ...
      'Units','normalized', ...
      'Position',[0.05 0.05 0.60 0.90], ...
      'Visible','off', ...
      'NextPlot','add');
   
   %===================================
   % Information for all buttons
   top=0.95;
   bottom=0.05;
   labelColor=[0.8 0.8 0.8];
   yInitPos=0.90;
   left=0.70;
   btnWid=0.25;
   btnHt=0.10;
   % Spacing between the button and the next command's label
   spacing=0.02;
   
   %====================================
   % The CONSOLE frame
   frmBorder=0.02;
   frmPos=[left-frmBorder bottom-frmBorder ...
           btnWid+2*frmBorder 0.9+2*frmBorder];
   h=uicontrol( ...
      'Style','frame', ...
      'Units','normalized', ...
      'Position',frmPos, ...
      'BackgroundColor',[0.5 0.5 0.5]);
   
   %====================================
   % The FLY button
   btnNumber=1;
   yPos=top-btnHt-(btnNumber-1)*(btnHt+spacing);
   labelStr='Info';
   cmdStr='info';
   callbackStr='neur_cellguih(''plot'');';
   
   % Generic button information
   btnPos=[left yPos-spacing btnWid btnHt];
   flyHndl=uicontrol( ...
      'Style','pushbutton', ...
      'Units','normalized', ...
      'Position',btnPos, ...
      'String',labelStr, ...
      'Callback',callbackStr);
   
   %====================================
   % The SELECT DEND popup button
   btnNumber=2;
   yPos=top-btnHt-(btnNumber-1)*(btnHt+spacing);
   labelStr='Record/Edit...';
   
   load ncmd nc md;
   data.nc = nc;
   data.md = md;
    
   popupStr1 = 'nothing selected';
   
   for ii = 1:md.count.N
       
       popupStr1 = str2mat(popupStr1,['Dendrite ' num2str(ii)]);
       
   end
     
   % Generic button information
   btnPos1=[left yPos-spacing+btnHt/2 btnWid btnHt/2];
   btnPos2=[left yPos-spacing btnWid btnHt/2];
   
   callbackStr = 'neur_cellguih(''dendchoose'')';
   
   uicontrol( ...
      'Style','text', ...
      'Units','normalized', ...
      'Position',btnPos1, ...
      'String',labelStr);
   popupHndl1=uicontrol( ...
      'Style','popup', ...
      'Units','normalized', ...
      'Position',btnPos2, ...
      'String',popupStr1, ...
      'CallBack',callbackStr, ...
      'UserData',data);
   
   %====================================
   % The SELECT DEND GROUP popup button
   btnNumber=3;
   yPos=top-btnHt-(btnNumber-1)*(btnHt+spacing);
   labelStr='Visualize...';
   
   % Generic button information
   btnPos1=[left yPos-spacing+btnHt/2 btnWid btnHt/2];
   btnPos2=[left yPos-spacing btnWid btnHt/2];
   
   callbackStr = 'neur_cellguih(''drawchoose'')';
   popupStr2 = str2mat('Soma','Roots','Terminals','Parents');
   val = [0 0 1 1]; 
  
   uicontrol( ...
      'Style','text', ...
      'Units','normalized', ...
      'Position',btnPos1, ...
      'String',labelStr);
   popupHndl2=uicontrol( ...
      'Style','popup', ...
      'Units','normalized', ...
      'Position',btnPos2, ...
      'String',popupStr2, ...
      'Value',1,...
      'CallBack',callbackStr,...
      'UserData',val);
   
   %====================================
   % The LABEL DENDS radio button
   btnNumber=4;
   yPos=top-btnHt-(btnNumber-1)*(btnHt+spacing);
   labelStr='Label dendrites';
   callbackStr='neur_cellguih(''labels'');';
   
   % Generic button information
   btnPos=[left yPos-spacing btnWid btnHt];
   westHndl=uicontrol( ...
      'Style','radiobutton', ...
      'Units','normalized', ...
      'Position',btnPos, ...
      'String',labelStr, ...
      'Value',0, ...
      'Callback',callbackStr);
   
   %====================================
   % The SIMPLE VIEW radio button
   btnNumber=5;
   yPos=top-btnHt-(btnNumber-1)*(btnHt+spacing);
   labelStr='Simple view';
   callbackStr='neur_cellguih(''schematic'');';
   
   % Generic button information
   btnPos=[left yPos-spacing btnWid btnHt];
   eastHndl=uicontrol( ...
      'Style','radiobutton', ...
      'Units','normalized', ...
      'Position',btnPos, ...
      'String',labelStr, ...
      'Value',0, ...
      'Callback',callbackStr);
   
   %====================================
   % The EDIT button
   labelStr='Edit';
   callbackStr='';
   infoHndl=uicontrol( ...
      'Style','push', ...
      'Units','normalized', ...
      'Position',[left bottom+btnHt+spacing btnWid btnHt], ...
      'String',labelStr, ...
      'Callback',callbackStr);
   
   %====================================
   % The RECORD button
   labelStr='Record';
   callbackStr='neur_cellguih(''record'')';
   closeHndl=uicontrol( ...
      'Style','push', ...
      'Units','normalized', ...
      'Position',[left bottom btnWid btnHt], ...
      'String',labelStr, ...
      'Callback',callbackStr);
   
   % Uncover the figure
   hndlList=[popupHndl1 popupHndl2 westHndl eastHndl];
   set(figNumber, ...
      'Visible','on', ...
      'UserData',hndlList);
   
   neur_cellgui2(nc,md,val)
   
   watchoff(oldFigNumber);
   figure(figNumber);
   
elseif strcmp(action,'plot'),
   %====================================
   axHndl=gca;
   figNumber=watchon;
   hndlList=get(figNumber,'Userdata');
   popupHndl1=hndlList(1);
   popupHndl2=hndlList(2);
   westHndl=hndlList(3);
   eastHndl=hndlList(4);
   
   %neur_cellgui2('');
   
   watchoff(figNumber);
   
elseif strcmp(action,'labels'),
   axHndl=gca;
   figNumber=watchon;
   drawnow;
   hndlList=get(figNumber,'Userdata');
   popupHndl1=hndlList(1);
   popupHndl2=hndlList(2);
   westHndl=hndlList(3);
   eastHndl=hndlList(4);
   watchoff(figNumber);
   
   data = get(popupHndl1,'UserData');
   val = get(popupHndl2,'UserData');
   res = get(westHndl,'Value');
   
   val(1) = res;
   set(popupHndl2,'UserData',val);
   
   neur_cellgui2(data.nc,data.md,val)
   
elseif strcmp(action,'schematic'),
   axHndl=gca;
   figNumber=watchon;
   drawnow;
   hndlList=get(figNumber,'Userdata');
   popupHndl1=hndlList(1);
   popupHndl2=hndlList(2);
   westHndl=hndlList(3);
   eastHndl=hndlList(4);
   watchoff(figNumber);   
   
   data = get(popupHndl1,'UserData');
   val = get(popupHndl2,'UserData');
   res = get(eastHndl,'Value');
   
   val(2) = res;
   set(popupHndl2,'UserData',val);
   
   neur_cellgui2(data.nc,data.md,val)


elseif strcmp(action,'drawchoose'),
   axHndl=gca;
   figNumber=watchon;
   drawnow;
   hndlList=get(figNumber,'Userdata');
   popupHndl1=hndlList(1);
   popupHndl2=hndlList(2);
   westHndl=hndlList(3);
   eastHndl=hndlList(4);
   watchoff(figNumber);   
   
   data = get(popupHndl1,'UserData');
   val = get(popupHndl2,'UserData');
   res = get(popupHndl2,'Value');
   
   val(3) = res;
   set(popupHndl2,'UserData',val);
   
   neur_cellgui2(data.nc,data.md,val)   

elseif strcmp(action,'dendchoose'),
   axHndl=gca;
   figNumber=watchon;
   drawnow;
   hndlList=get(figNumber,'Userdata');
   popupHndl1=hndlList(1);
   popupHndl2=hndlList(2);
   westHndl=hndlList(3);
   eastHndl=hndlList(4);
   watchoff(figNumber);   
   
   data = get(popupHndl1,'UserData');
   val = get(popupHndl2,'UserData');
   res = get(popupHndl1,'Value');
   
   val(4) = res;
   set(popupHndl2,'UserData',val);
   
   neur_cellgui2(data.nc,data.md,val)      
   
elseif strcmp(action,'record'),
   
    axHndl=gca;
    figNumber=watchon;
    drawnow;
    
    hndlList=get(figNumber,'Userdata');
    popupHndl1=hndlList(1);
    popupHndl2=hndlList(2);
    westHndl=hndlList(3);
    eastHndl=hndlList(4);
    
    dendnum = get(popupHndl1,'Value')-1;
    data = get(popupHndl1,'UserData');
    
    if dendnum > 0
        vdat = orderpotsh(data.md,data.nc,dendnum);
        neur_recguih('init',dendnum,vdat,data.nc,data.md);
    end
    
    watchoff(figNumber);
        
end;    % if strcmp(action, ...

%  Restore Format
set(0,'Format',old_format)

function neur_cellgui2(nc,md,val);

set(gca, ...
    'NextPlot','add', ...
    'Visible','off',...
    'Color',[1 1 1]);
axis equal
set(get(gca,'Title'),'Visible','on')

cla
neur_plotcell(nc,md,val)

function neur_plotcell(nc,md,val)

segcount = length(nc.celldata);

hold on

terms = repmat(1,1,md.count.N);
terms(md.par) = 0;
termdex = find(terms == 1);

switch(val(3))
   
    case 1
        pchro = 'bbbr';
    case 2
        pchro = 'bbrb';
    case 3
        pchro = 'brbb';
    case 4
        pchro = 'rbbb';
end

pchro(5) = 'y';

if ~val(2)
    
    for jj = 1:length(md.par)
        
        ii = md.par(jj);
        
        plot3(nc.celldata{ii}(:,1),...
              nc.celldata{ii}(:,2),...
              nc.celldata{ii}(:,3),...
              pchro(1),...
              'LineWidth',ceil(5*nc.celldata{ii}(1,4)/md.maxrad));
    end
    
    for jj = 1:length(termdex)
        
        ii = termdex(jj);
        
        plot3(nc.celldata{ii}(:,1),...
              nc.celldata{ii}(:,2),...
              nc.celldata{ii}(:,3),...
              pchro(2),...
              'LineWidth',ceil(5*nc.celldata{ii}(1,4)/md.maxrad));
    end
    
    for jj = 1:length(nc.roots)
        
        ii = nc.roots(jj);
        
        plot3(nc.celldata{ii}(:,1),...
              nc.celldata{ii}(:,2),...
              nc.celldata{ii}(:,3),...
              pchro(3),...
              'LineWidth',ceil(5*nc.celldata{ii}(1,4)/md.maxrad));
    end
    
    patch(nc.somadata.x,...
          nc.somadata.y,...
          nc.somadata.z,pchro(4))
    
% schematic mode    

else

    for jj = 1:length(md.par)
        
        ii = md.par(jj);
     
        h = line([nc.celldata{ii}(1,1) nc.celldata{ii}(end,1)],...
                 [nc.celldata{ii}(1,2) nc.celldata{ii}(end,2)],...
                 [nc.celldata{ii}(1,3) nc.celldata{ii}(end,3)]);
             
        %set(h,'Color',pchro(1));
    end
    
    for jj = 1:length(termdex)
        
        ii = termdex(jj);
    
        h = line([nc.celldata{ii}(1,1) nc.celldata{ii}(end,1)],...
                 [nc.celldata{ii}(1,2) nc.celldata{ii}(end,2)],...
                 [nc.celldata{ii}(1,3) nc.celldata{ii}(end,3)]);
             
        %set(h,'Color',pchro(2));
    end
    
    for jj = 1:length(nc.roots)
        
        ii = nc.roots(jj);
    
        h = line([nc.celldata{ii}(1,1) nc.celldata{ii}(end,1)],...
                 [nc.celldata{ii}(1,2) nc.celldata{ii}(end,2)],...
                 [nc.celldata{ii}(1,3) nc.celldata{ii}(end,3)]);
             
        %set(h,'Color',pchro(3));
    end
    
    patch(nc.somadata.x,...
          nc.somadata.y,...
          nc.somadata.z,'r')

end

xlim = get(gca,'XLim');
ylim = get(gca,'YLim');

mlen = round(1/8*(xlim(2)-xlim(1))*1e-2)*1e2;

xp = [xlim(1) xlim(1)+mlen];
yp1 = [ylim(2)-1/8*(ylim(2)-ylim(1)) ylim(2)-1/8*(ylim(2)-ylim(1))];
yp2 = [ylim(2)-3/16*(ylim(2)-ylim(1)) ylim(2)-3/16*(ylim(2)-ylim(1))];

%mlen = 1/4*diff(xlim)*1e-4/md.grid.hstep(end);

plot(xp,yp1,'c')
mscale = text(xp(1),yp2(1),[num2str(mlen) '\mum'])
set(mscale,'Color','Cyan','Fontsize',12)

% plot the soma

% lastindex = length(nc.somadata.x);

%plot3([nc.somadata.x(1) nc.somadata.x(lastindex)],...
%      [nc.somadata.y(1) nc.somadata.y(lastindex)],...
%      [nc.somadata.z(1) nc.somadata.z(lastindex)],pchro(4))

  
drawnow      
      
% highlight dendrite

dn = val(4)-1;

if dn > 0
    if ~val(2)

        plot3(nc.celldata{dn}(:,1),...
              nc.celldata{dn}(:,2),...
              nc.celldata{dn}(:,3),...
              pchro(5),...
              'LineWidth',ceil(5*nc.celldata{dn}(1,4)/md.maxrad));

        % schematic mode

    else

        plot3([nc.celldata{dn}(1,1) nc.celldata{dn}(end,1)],...
              [nc.celldata{dn}(1,2) nc.celldata{dn}(end,2)],...
              [nc.celldata{dn}(1,3) nc.celldata{dn}(end,3)],...
              pchro(5));

    end
end

% label dendrites  
      
if val(1)
    plotsegnum(nc)
end
  
function plotsegnum(nc)

segcount = length(nc.celldata);

for i = 1:segcount
    hold on
    h = text(nc.celldata{i}(1,1),...
             nc.celldata{i}(1,2),...
             nc.celldata{i}(1,3),num2str(i));
         
    set(h,...
        'Color','cyan',...
        'FontSize',8);
end

function vdat = orderpotsh(md,nc,dendnum)

disp('loading potentials...')
%load nc
%keyboard
%out = neur_read(nc.filename);
load out

%keyboard

% if find(dendnum == nc.roots(end))
%     
%     vdex = [md.segt(dendnum) md.segh(dendnum)];
%     vdat.v = out.v([vdex(1):-1:vdex(2)],:);
%     
% elseif find(nc.roots == dendnum) 
%     
%     vdex = [md.count.L md.segt(dendnum) md.segh(dendnum)];
%     vdat.v = out.v([vdex(1) vdex(2):-1:vdex(3)],:);
%     
% else
%     
%     [par,chi] = getparent(nc,md);
%     pardex = par(find(chi == dendnum));
%     vdex = [md.segh(pardex) md.segt(dendnum) md.segh(dendnum)] ;
%     vdat.v = out.v([vdex(1) vdex(2):-1:vdex(3)],:);    
% end

vdat.v = out.v{dendnum}(end:-1:1,:);
vdat.INa = out.INa{dendnum}(end:-1:1,:);
vdat.IK = out.IK{dendnum}(end:-1:1,:);
vdat.IL = out.IL{dendnum}(end:-1:1,:);
vdat.IA = out.IA{dendnum}(end:-1:1,:);
vdat.Ca = out.Ca{dendnum}(end:-1:1,:);
vdat.CaB = out.CaB{dendnum}(end:-1:1,:);
vdat.CaE = out.CaE{dendnum}(end:-1:1,:);
vdat.t = out.t;
%vdat.x = md.grid.x{dendnum}(end:-1:1);
vdat.x = out.x{dendnum};

function [parent,children] = getparent(nc,md)

k = 1;

parent = [];
children = [];

for ii=1:md.count.N

    if size(nc.treedata{ii}) > 0
        parent(k) = ii;
        children(k) = nc.treedata{ii}(1);
        k = k + 1;
        parent(k) = ii;
        children(k) = nc.treedata{ii}(2);
        k = k + 1;
    end
    
end
